package com.unissoft.config;

import com.unissoft.dto.ResourceRoleInfoDto;
import com.unissoft.service.SysResourceRoleService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.ConfigAttribute;
import org.springframework.security.access.SecurityConfig;
import org.springframework.security.web.FilterInvocation;
import org.springframework.security.web.access.intercept.FilterInvocationSecurityMetadataSource;
import org.springframework.stereotype.Component;
import org.springframework.util.AntPathMatcher;
import org.springframework.util.PathMatcher;

import java.util.*;
import java.util.stream.Collectors;

/**
 * 动态获取url权限配置
 */
@Component
public class MySecurityMetadataSource implements FilterInvocationSecurityMetadataSource {

    private LinkedHashMap<String, Collection<ConfigAttribute>> map = null;
    private PathMatcher pathMatcher = new AntPathMatcher();//URLs匹配类

    @Autowired
    private MyConfig myConfig;

    @Autowired
    private SysResourceRoleService resourceRoleMapper;

    /**
     * 加载权限表中所有操作请求权限
     */
    public void loadResourceDefine() {
        LinkedHashMap<String, Collection<ConfigAttribute>> linkedHashMap = new LinkedHashMap<>();
        //查当前项目的资源角色信息
        List<ResourceRoleInfoDto> dtos = resourceRoleMapper.getResourceRoleInfo(myConfig.getProjectCode());
        for (ResourceRoleInfoDto dto : dtos) {
            String urlPattern1 = dto.getButton();
            Collection<ConfigAttribute> configAttributes1 = new ArrayList<>();
            if (dto.getRoleId() == null){
                linkedHashMap.put(urlPattern1, configAttributes1);
            }else {
                SecurityConfig securityConfig = new SecurityConfig("" + dto.getRoleId());
                if (!linkedHashMap.containsKey(urlPattern1)) {
                    //角色信息2（对应urlPattern2）
                    Collection<ConfigAttribute> configAttributes2 = new ArrayList<>();
                    configAttributes2.add(securityConfig);
                    linkedHashMap.put(urlPattern1, configAttributes2);
                } else {
                    Collection<ConfigAttribute> configAttributes = linkedHashMap.get(urlPattern1);
                    if(!configAttributes.contains(securityConfig)){
                        configAttributes.add(securityConfig);
                    }
                }
            }
        }
        map = linkedHashMap;

        //下面是打印
        Iterator<Map.Entry<String, Collection<ConfigAttribute>>> iterator = linkedHashMap.entrySet().iterator();
        while (iterator.hasNext()) {
            Map.Entry<String, Collection<ConfigAttribute>> next = iterator.next();
            String key = next.getKey();
            Collection<ConfigAttribute> value = next.getValue();
            if (value.size() > 0) {
                System.err.println(key + " --- 路径下用户拥有这些角色id " + value.toString() + " 才可以访问！");
            } else {
                System.err.println(key + " --- 路径下所有用户不能访问！");
            }
        }
    }


    /**
     * 判定用户请求的url是否在权限表中
     * 如果在权限表中，则返回给MyAccessDecisionManager类decide方法，用来判定用户是否有此权限
     * 如果不在权限表中则放行
     *
     * @param o
     * @return
     * @throws IllegalArgumentException
     */
    @Override
    public Collection<ConfigAttribute> getAttributes(Object o) throws IllegalArgumentException {
        //Object中包含用户请求request url
        String resUrl = ((FilterInvocation) o).getRequestUrl();

        List<String> urls = map.keySet().stream().filter(url -> pathMatcher.match(url, resUrl)).collect(Collectors.toList());
        if (urls.size() > 0 && map.get(urls.get(0)).size() > 0) {
            return map.get(urls.get(0));
        } else {
            return null;
        }
    }

    @Override
    public Collection<ConfigAttribute> getAllConfigAttributes() {
        return null;
    }

    @Override
    public boolean supports(Class<?> aClass) {
        return true;
    }
}
