package com.unissoft.facilities.controller;


import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.unissoft.facilities.service.EquipFacilitiesService;
import com.unissoft.model.EquipFacilities;
import com.unissoft.model.EquipFacilitiesBase;
import com.unissoft.result.PageParam;
import com.unissoft.result.ResultView;
import io.swagger.annotations.ApiOperation;
import org.apache.ibatis.annotations.Param;
import org.springframework.data.domain.Page;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.validation.constraints.NotNull;
import java.util.List;

/**
 * <p>
 * 前端控制器
 * </p>
 *
 * @author root
 * @since 2021-01-25
 */
@RestController
@RequestMapping("/equipFacilities")
public class EquipFacilitiesController {
    @Resource
    private EquipFacilitiesService equipFacilitiesService;

    @ApiOperation(value = "新增", notes = "设施检查记录新增")
    @PostMapping("/facilities/add")
    public ResultView add(@RequestBody EquipFacilitiesBase model) {
        // 同时保存主表,子表信息
        boolean result = equipFacilitiesService.savePrimeAndChildren(model);
        if (result)
            return ResultView.success();
        return ResultView.error();
    }

    @ApiOperation(value = "修改", notes = "设施检查记录修改")
    @PostMapping("/facilities/update")
    public ResultView update(@RequestBody EquipFacilitiesBase model) {
        boolean result = equipFacilitiesService.updatePrimeAndChildren(model);
        if (result) {
            return ResultView.success();
        }
        return ResultView.error();
    }

    /**
     * 最新设施检查记录
     *
     * @param model
     * @return
     */
    @ApiOperation(value = "最新设施检查记录", notes = "所属库编码,仓房编码必传")
    @GetMapping("/facilities/lastFacilities")
    public ResultView lastFacilitiesList(EquipFacilities model) {
        List<EquipFacilities> list = equipFacilitiesService.getLastFacilitiesList(model);
        return ResultView.success(list);
    }

    /**
     * 全部设施检查记录分页数据
     *
     * @param pageParam
     * @return
     */
    @ApiOperation(value = "全部设施检查记录分页数据", notes = "分页数据")
    @GetMapping("/facilities/getPageInfo")
    @ResponseBody
    public ResultView getPageInfo(@Validated PageParam pageParam) {
        IPage<EquipFacilities> PageList = equipFacilitiesService.getPageInfo(pageParam);
        return ResultView.success(PageList);
    }

    /*@ApiOperation(value = "全部设施检查记录", notes = "单条数据")
    @GetMapping("/facilities/getById/{id}")
    public ResultView getById(@PathVariable("id") @NotNull Integer id) {
        EquipFacilities equipFacilities = equipFacilitiesService.getById(id);
        return ResultView.success(equipFacilities);
    }*/

    @ApiOperation(value = "全部设施检查记录", notes = "删除")
    @DeleteMapping("/facilities/deleteById/{id}")
    public ResultView deleteById(@PathVariable("id") @NotNull Integer id) {
        if (equipFacilitiesService.deleteById(id))
            return ResultView.success();
        return ResultView.error();
    }

}
