package com.unissoft.facilities.service.impl;

import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.unissoft.constant.GrainConstant;
import com.unissoft.facilities.service.EquipFacilitiesService;
import com.unissoft.mapper.EquipFacilitiesChildMapper;
import com.unissoft.mapper.EquipFacilitiesMapper;
import com.unissoft.model.EquipFacilities;
import com.unissoft.model.EquipFacilitiesBase;
import com.unissoft.model.EquipFacilitiesChild;
import com.unissoft.result.PageParam;
import io.jsonwebtoken.lang.Collections;
import org.json.JSONObject;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.StringUtils;
import org.springframework.validation.annotation.Validated;

import javax.annotation.Resource;
import java.util.List;
import java.util.Optional;

/**
 * <p>
 * 服务实现类
 * </p>
 *
 * @author root
 * @since 2021-01-25
 */
@Service
public class EquipFacilitiesServiceImpl extends ServiceImpl<EquipFacilitiesMapper, EquipFacilities> implements EquipFacilitiesService {
    @Resource
    private EquipFacilitiesMapper equipFacilitiesMapper;
    @Resource
    private EquipFacilitiesChildMapper facilitiesChildMapper;

    @SuppressWarnings("all")
    @Override
    public List<EquipFacilities> getLastFacilitiesList(EquipFacilities model) {
        // 获取 最新设施检查记录
        List<EquipFacilities> list = equipFacilitiesMapper.selectByLastFacilitiesList(model);
        if (list.size() < GrainConstant.KIND_LONG || Collections.isEmpty(list)) {
            /*
             * 检查补齐初始化数据
             */
            // 仓房设施
            if (!list.stream().anyMatch(e -> e.getKind() == GrainConstant.KIND_41)) {
                EquipFacilities eq = new EquipFacilities();
                eq.setKind(GrainConstant.KIND_41);
                list.add(eq);
            }
            // 消防设施
            if (!list.stream().anyMatch(e -> e.getKind() == GrainConstant.KIND_42)) {
                EquipFacilities eq = new EquipFacilities();
                eq.setKind(GrainConstant.KIND_42);
                list.add(eq);
            }
            // 计量设施
            if (!list.stream().anyMatch(e -> e.getKind() == GrainConstant.KIND_43)) {
                EquipFacilities eq = new EquipFacilities();
                eq.setKind(GrainConstant.KIND_43);
                list.add(eq);
            }
            // 药品库设施
            if (!list.stream().anyMatch(e -> e.getKind() == GrainConstant.KIND_44)) {
                EquipFacilities eq = new EquipFacilities();
                eq.setKind(GrainConstant.KIND_44);
                list.add(eq);
            }
            // 配电室设施
            if (!list.stream().anyMatch(e -> e.getKind() == GrainConstant.KIND_45)) {
                EquipFacilities eq = new EquipFacilities();
                eq.setKind(GrainConstant.KIND_45);
                list.add(eq);
            }
        }
        // 数据不足或初始化数据补齐
        return list;
    }

    @Transactional
    @Override
    public boolean savePrimeAndChildren(EquipFacilitiesBase model) {
        // 非空
        if (Optional.ofNullable(model).isPresent()) {
            // 主表信息
            EquipFacilities equipFacilities = model.getEquipFacilities();
            // 子表信息
            List<EquipFacilitiesChild> equipFacilitiesChild = model.getEquipFacilitiesChildren();
            if (equipFacilitiesMapper.insert(equipFacilities) > 0) {
                // 获取主表id,保存到子表pId关联字段
                Integer id = equipFacilities.getId();
                equipFacilitiesChild.forEach(f -> {
                    f.setPId(id);
                    facilitiesChildMapper.insert(f);
                });
                return true;
            }
            return false;
        }
        return false;
    }

    @Override
    public IPage<EquipFacilities> getPageInfo(PageParam pageParam) {
        // 查询对象
        QueryWrapper<EquipFacilities> query = new QueryWrapper<>();
        if (!StringUtils.isEmpty(pageParam.getCondition())) {
            JSONObject object = new JSONObject(pageParam.getCondition());
            String kind = object.optString("kind").trim(); // 种类
            String startTime = object.optString("startTime").trim(); // 开始时间
            String endTime = object.optString("endTime").trim(); // 结束时间
            String houseId = object.optString("hId").trim();
            String depotId = object.optString("depotId").trim();
            if (!StringUtils.isEmpty(kind)) {
                query.eq("kind", kind);
            }
            if (!StringUtils.isEmpty(startTime)) {
                query.ge("check_time", startTime);
            }
            if (!StringUtils.isEmpty(endTime)) {
                query.le("check_time", endTime);
            }
            if (!StringUtils.isEmpty(houseId)) {
                query.eq("h_id", houseId);
            }
            if (!StringUtils.isEmpty(depotId)) {
                query.eq("depot_id", depotId);
            }
        }
        query.orderByDesc("check_time");
        // 分页参数
        IPage<EquipFacilities> iPage = new Page<>();
        iPage.setCurrent(pageParam.getPageIndex());
        iPage.setSize(pageParam.getPageSize());
        // 查询分页数据
        return equipFacilitiesMapper.selectPage(iPage, query);
    }

    @Transactional
    @Override
    public boolean deleteById(Integer id) {
        if (equipFacilitiesMapper.deleteById(id) > 0) {
            QueryWrapper<EquipFacilitiesChild> childQueryWrapper = new QueryWrapper<>();
            childQueryWrapper.eq("p_id", id);
            facilitiesChildMapper.delete(childQueryWrapper);
            return true;
        }
        return false;
    }

    @Transactional
    @Override
    public boolean updatePrimeAndChildren(EquipFacilitiesBase model) {
        // 非空
        if (Optional.ofNullable(model).isPresent()) {
            // 主表信息
            EquipFacilities equipFacilities = model.getEquipFacilities();
            // 子表信息
            List<EquipFacilitiesChild> equipFacilitiesChild = model.getEquipFacilitiesChildren();
            if (equipFacilitiesMapper.updateById(equipFacilities) > 0) {
                // 根据id更新
                equipFacilitiesChild.forEach(f -> facilitiesChildMapper.updateById(f));
                return true;
            }
            return false;
        }
        return false;
    }
}
