package com.unissoft.grainManage.controller;


import com.baomidou.mybatisplus.core.metadata.IPage;
import com.unissoft.constant.GrainConstant;
import com.unissoft.grainManage.service.GrainConditionRecordService;
import com.unissoft.model.GrainAdd;
import com.unissoft.model.GrainConditionRecord;
import com.unissoft.model.GrainError;
import com.unissoft.model.GrainErrorEChars;
import com.unissoft.result.PageParam;
import com.unissoft.result.ResultView;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.validation.constraints.NotNull;
import java.util.List;
import java.util.Map;

/**
 * <p>
 * 前端控制器
 * </p>
 *
 * @author root
 * @since 2020-12-04
 */
@Slf4j
@Api(description = "粮情检测记录")
@RestController
@RequestMapping("/grainConditionRecord")
public class GrainConditionRecordController {
    @Resource
    private GrainConditionRecordService grainConditionRecordService;

    @ApiOperation(value = "最新粮情检查记录", notes = "获取最新粮情检查记录或初始化")
    @PostMapping("/authority/getByRoleTime")
    public ResultView getByRoleTime(@RequestBody GrainConditionRecord record) {
        if (null == record.getDepotId() || null == record.getHouseId()) {
            return ResultView.error(GrainConstant.DEPOT_HOUSE_NOT_NULL);
        }
        List<GrainConditionRecord> list = grainConditionRecordService.getByRoleTime(record);
        return ResultView.success(list);
    }

    @ApiOperation(value = "粮情检测记录新增", notes = "新增主表和子表数据")
    @PostMapping("/authority_button/add")
    public ResultView add(@RequestBody GrainAdd model) {
        if (grainConditionRecordService.saveGrainRecord(model)) {
            return ResultView.success();
        }
        return ResultView.error();
    }

    @ApiOperation(value = "修改", notes = "修改主表和子表数据")
    @PostMapping("/authority_button/update")
    public ResultView update(@RequestBody GrainAdd model) {
        if(grainConditionRecordService.updateGrainRecord(model)) {
            return ResultView.success();
        }
        return ResultView.error();
    }

    @ApiOperation(value = "分页条件查询", notes = "提交参数: ")
    @GetMapping("/authority/getPage")
    public ResultView getPage(@Validated PageParam pageParam) {
        IPage<GrainConditionRecord> page = grainConditionRecordService.getPage(pageParam);
        return ResultView.success(page);
    }

    @ApiOperation(value = "删除检查记录", notes = "删除")
    @DeleteMapping("/authority/deleteById/{id}")
    public ResultView deleteById(@PathVariable("id") @NotNull Integer id) {
        if (grainConditionRecordService.deleteById(id))
            return ResultView.success();
        return ResultView.error();
    }

    @ApiOperation(value = "首页粮情异常eChars", notes = "图表")
    @GetMapping("/authority/getGrainErrorEChars/{depotId}")
    public ResultView getGrainErrorEChars(@PathVariable("depotId") Integer depotId) {
        List<GrainErrorEChars> errorList = grainConditionRecordService.getGrainErrorEChars(depotId);
        return ResultView.success(errorList);
    }

    @ApiOperation(value = "粮情异常列表分页查询", notes = "提交参数: depotId 非空")
    @PostMapping("/authority/getGrainErrorCount")
    public ResultView getGrainErrorCount(@RequestBody PageParam pageParam) {
        IPage<GrainError> page = grainConditionRecordService.getGrainErrorCount(pageParam);
        return ResultView.success(page);
    }

    @ApiOperation(value = "本机构异常粮情列表分页查询", notes = "提交参数: depotId 非空")
    @PostMapping("/authority/getGrainErrorHouseList")
    public ResultView getGrainErrorHouseList(@RequestBody PageParam pageParam) {
        IPage<GrainError> page = grainConditionRecordService.getGrainErrorHouseList(pageParam);
        return ResultView.success(page);
    }

}
