package com.unissoft.grainManage.service.impl;

import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.unissoft.MyConstant;
import com.unissoft.constant.GrainConstant;
import com.unissoft.feign.SystemFeign;
import com.unissoft.grainManage.service.GrainConditionRecordService;
import com.unissoft.mapper.*;
import com.unissoft.model.*;
import com.unissoft.result.PageParam;
import com.unissoft.utils.DateUtil;
import lombok.extern.slf4j.Slf4j;
import org.json.JSONObject;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import javax.annotation.Resource;
import java.util.*;

/**
 * <p>
 * 服务实现类
 * </p>
 *
 * @author root
 * @since 2020-12-04
 */
@Slf4j
@Service
@SuppressWarnings("all")
public class GrainConditionRecordServiceImpl extends ServiceImpl<GrainConditionRecordMapper, GrainConditionRecord> implements GrainConditionRecordService {

    @Autowired
    private SysResourceRoleMapper resourceRoleMapper;
    @Autowired
    private GrainConditionRecordMapper grainConditionRecordMapper;
    @Autowired
    private GrainConditionRecordChildMapper childMapper;
    @Resource
    private SystemFeign systemFeign;

    @Override
    public List<GrainConditionRecord> getByRoleTime(GrainConditionRecord requestRecord) {
        log.info("GrainConditionRecord request param: {}", JSON.toJSONString(requestRecord));
        // 查询数据
        List<GrainConditionRecord> list = grainConditionRecordMapper.getByCheckPersonAndCheckTime(requestRecord.getHouseId(), requestRecord.getDepotId());

        // 前端初始值,判断缺哪个补哪个
        if (CollectionUtils.isEmpty(list) || list.size() < 4) {
            // 主管科长每两周检查粮情
            if (!list.stream().anyMatch(element -> element.getEnumId().equals(GrainConstant.SECTION_CHIEF))) {
                GrainConditionRecord record = new GrainConditionRecord();
                record.setEnumId(GrainConstant.SECTION_CHIEF);
                list.add(record);
            }
            // 保管员每周检查粮情
            if (!list.stream().anyMatch(element -> element.getEnumId().equals(GrainConstant.GUARDIAN))) {
                GrainConditionRecord record = new GrainConditionRecord();
                record.setEnumId(GrainConstant.GUARDIAN);
                list.add(record);
            }
            // 分管副总每月检查粮情
            if (!list.stream().anyMatch(element -> element.getEnumId().equals(GrainConstant.DEPUTY_GENERAL))) {
                GrainConditionRecord record = new GrainConditionRecord();
                record.setEnumId(GrainConstant.DEPUTY_GENERAL);
                list.add(record);
            }
            // 总经理每季检查粮情
            if (!list.stream().anyMatch(element -> element.getEnumId().equals(GrainConstant.GENERAL_MANAGER))) {
                GrainConditionRecord record = new GrainConditionRecord();
                record.setEnumId(GrainConstant.GENERAL_MANAGER);
                list.add(record);
            }
        }
        log.info("getByRoleTime 返回数据: {}", list);
        return list;
    }

    @Transactional
    @Override
    public boolean saveGrainRecord(GrainAdd model) {
        // 新增粮情检查 主表数据
        GrainConditionRecord record = model.getGrainConditionRecord();
        // 处理接收到的主表数据
        saveAndUpdateCommon(record);
        // 主表保存成功
        if (grainConditionRecordMapper.insert(record) > 0) {
            // 获取主表自增id
            Integer id = record.getId();
            List<GrainConditionRecordChild> children = model.getGrainConditionRecordChildren();
            children.forEach(c -> {
                c.setParentId(id);
                childMapper.insert(c);
            });
            return true;
        }
        return false;
    }

    @Transactional
    @Override
    public boolean updateGrainRecord(GrainAdd model) {
        // 修改主表数据
        GrainConditionRecord record = model.getGrainConditionRecord();
        // 处理接收到的主表数据
        saveAndUpdateCommon(record);
        // 主表修改成功
        if (grainConditionRecordMapper.updateById(record) > 0) {
            // 修改子表数据
            List<GrainConditionRecordChild> children = model.getGrainConditionRecordChildren();
            children.forEach(cu -> {
                childMapper.updateById(cu);
            });
            return true;
        }
        return false;
    }

    private void saveAndUpdateCommon(GrainConditionRecord record) {
        Date operationTime = record.getOperationTime();
        // 获取时间年
        String yyyy = DateUtil.getYMDHMS(operationTime, "yyyy");
        record.setCheckYear(Integer.parseInt(yyyy));
        // 获取时间所在年的第多少周(前端和后台获取到的周不一致,选择使用前端数据)
        /*Integer presentWeek = DateUtil.getPresentWeek(operationTime);
        record.setCheckYearWeek(presentWeek);*/
    }

    @Override
    public IPage<GrainConditionRecord> getPage(PageParam pageParam) {
        int pageIndex = pageParam.getPageIndex();
        int pageSize = pageParam.getPageSize();
        QueryWrapper<GrainConditionRecord> ew = new QueryWrapper<>();
        if (!StringUtils.isEmpty(pageParam.getCondition())) {
            JSONObject object = new JSONObject(pageParam.getCondition());
            String enumId = object.optString("enumId").trim();
            String startTime = object.optString("startTime").trim();
            String endTime = object.optString("endTime").trim();
            String houseId = object.optString("houseId").trim();
            String depotId = object.optString("depotId").trim();
            if (!StringUtils.isEmpty(enumId)) {
                ew.eq("enum_id", enumId);
            }
            if (!StringUtils.isEmpty(startTime)) {
                ew.ge("operation_time", startTime);
            }
            if (!StringUtils.isEmpty(endTime)) {
                ew.le("operation_time", endTime);
            }
            if (!StringUtils.isEmpty(houseId)) {
                ew.eq("house_id", houseId);
            }
            if (!StringUtils.isEmpty(depotId)) {
                ew.eq("depot_id", depotId);
            }
        }
        ew.orderByDesc("operation_time");
        IPage<GrainConditionRecord> iPage = new Page<>();
        if (pageIndex != MyConstant.ZERO && pageSize != MyConstant.ZERO) {
            iPage.setCurrent(pageIndex);
            iPage.setSize(pageSize);
        }
        return grainConditionRecordMapper.selectPage(iPage, ew);
    }

    @Transactional
    @Override
    public boolean deleteById(Integer id) {
        if (grainConditionRecordMapper.deleteById(id) > 0) {
            QueryWrapper<GrainConditionRecordChild> query = new QueryWrapper<>();
            query.eq("parent_id", id);
            if (childMapper.delete(query) > 0) {
                return true;
            }
        }
        return false;
    }

    @Override
    public List<GrainErrorEChars> getGrainErrorEChars(Integer depotId) {
        // 调用基础服务查询当前登录人所在机构及下级机构编码
        List<Integer> depotIds = (List) systemFeign.getItselfJoinDownDepotId(depotId).getData();
        return grainConditionRecordMapper.getGrainErrorEChars(depotIds);
    }

    @Override
    public IPage<GrainError> getGrainErrorCount(PageParam pageParam) {
        // 分页
        int pageIndex = pageParam.getPageIndex();
        int pageSize = pageParam.getPageSize();
        // 分页对象
        IPage<GrainError> iPage = new Page<>();
        // 查询条件
        if (!StringUtils.isEmpty(pageParam.getCondition())) {
            JSONObject object = new JSONObject(pageParam.getCondition());
            Integer depotId = object.getInt("depotId"); // 库id
            // 调用基础服务查询当前登录人所在机构及下级机构编码
            List<Integer> depotIds = (List) systemFeign.getItselfJoinDownDepotId(depotId).getData();
            log.info("depotIds -> {}", JSON.toJSONString(depotIds));
            if (pageIndex != MyConstant.ZERO && pageSize != MyConstant.ZERO) {
                iPage.setCurrent(pageIndex);
                iPage.setSize(pageSize);
            }
            // 查询map对象
            Map<String, Object> map = JSON.parseObject(pageParam.getCondition(), Map.class);
            map.put("depotIds", depotIds);
            // 获取分页查询数据
            IPage<GrainError> res = grainConditionRecordMapper.getGrainErrorCount(iPage, map);
            log.info("IPage res-> {}", JSON.toJSONString(res));
            return res;
        }
        return iPage;
    }

    @Override
    public IPage<GrainError> getGrainErrorHouseList(PageParam pageParam) {
        // 分页
        int pageIndex = pageParam.getPageIndex();
        int pageSize = pageParam.getPageSize();
        // 分页对象
        IPage<GrainError> iPage = new Page<>();
        // 查询条件
        if (!StringUtils.isEmpty(pageParam.getCondition())) {
            if (pageIndex != MyConstant.ZERO && pageSize != MyConstant.ZERO) {
                iPage.setCurrent(pageIndex);
                iPage.setSize(pageSize);
            }
            // 查询map对象
            Map<String, Object> map = JSON.parseObject(pageParam.getCondition(), Map.class);
            // 获取分页查询数据
            IPage<GrainError> res = grainConditionRecordMapper.getGrainErrorHouseList(iPage, map);
            return res;
        }
        return iPage;
    }

}
