package com.unissoft.grainManage.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.unissoft.MyConstant;
import com.unissoft.grainManage.service.GrainSecurityInspectDetailsService;
import com.unissoft.grainManage.service.GrainSecurityInspectService;
import com.unissoft.mapper.GrainSecurityInspectMapper;
import com.unissoft.model.GrainSecurityInspect;
import com.unissoft.model.GrainSecurityInspectDetails;
import com.unissoft.model.SysUserPO;
import com.unissoft.result.PageParam;
import com.unissoft.utils.DateUtil;
import com.unissoft.vo.GrainSecurityInspectVO;
import org.apache.commons.lang.StringUtils;
import org.apache.commons.lang.time.DateUtils;
import org.json.JSONObject;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.*;

/**
 * <p>
 * 安全巡查 服务实现类
 * </p>
 *
 * @author root
 * @since 2021-01-18
 */
@Service
public class GrainSecurityInspectServiceImpl extends ServiceImpl<GrainSecurityInspectMapper, GrainSecurityInspect> implements GrainSecurityInspectService {

    private static final String DAY_INSPECT = "1";
    private static final String WEEK_INSPECT = "2";
    private static final String MONTH_INSPECT = "3";

    @Autowired
    private GrainSecurityInspectMapper grainSecurityInspectMapper;

    @Autowired
    private GrainSecurityInspectDetailsService grainSecurityInspectDetailsService;

    @Override
    public List<GrainSecurityInspect> getEveryTypeNewData(Integer depotId) {
        List<GrainSecurityInspect> result = new ArrayList<>();
        //
        GrainSecurityInspect dayInspect = getNewDataByType(depotId, DAY_INSPECT);
        result.add(dayInspect);
        //
        GrainSecurityInspect weekInspect = getNewDataByType(depotId, WEEK_INSPECT);
        result.add(weekInspect);
        //
        GrainSecurityInspect monthInspect = getNewDataByType(depotId, MONTH_INSPECT);
        result.add(monthInspect);
        //
        return result;
    }

    private GrainSecurityInspect getNewDataByType(Integer depotId, String inspecType) {
        GrainSecurityInspect result = null;
        Map<String, Object> param = new HashMap<>();
        param.put("depotId", depotId);
        param.put("inspecType", inspecType);
        List<GrainSecurityInspect> dayInspectList = grainSecurityInspectMapper.getNewDataByType(param);
        if(null != dayInspectList && dayInspectList.isEmpty() == false) {
            result = dayInspectList.get(0);
        } else {
            result = new GrainSecurityInspect();
            result.setInspecType(inspecType);
        }
        return result;
    }


    @Override
    public Page<GrainSecurityInspect> getAllPage(Integer depotId, PageParam pageParam) {
        Integer pageIndex = pageParam.getPageIndex();
        Integer pageSize = pageParam.getPageSize();
        QueryWrapper<GrainSecurityInspect> ew = new QueryWrapper<>();
        ew.ge("depot_id", depotId);
        if(! StringUtils.isEmpty(pageParam.getCondition())){
            JSONObject object = new JSONObject(pageParam.getCondition());

            if (object.has("inspecTimeStart")) {
                String inspecTimeStartStr = object.getString("inspecTimeStart");
                if (! StringUtils.isEmpty(inspecTimeStartStr)) {
                    Date inspecTimeStart = DateUtil.formatDate(inspecTimeStartStr, "yyyy-MM-dd");
                    ew.ge("inspec_time", inspecTimeStart);
                }
            }

            if (object.has("inspecTimeEnd")) {
                String inspecTimeEndStr = object.getString("inspecTimeEnd");
                if (! StringUtils.isEmpty(inspecTimeEndStr)) {
                    Date inspecTimeEnd = DateUtil.formatDate(inspecTimeEndStr, "yyyy-MM-dd");
                    inspecTimeEnd = DateUtils.addDays(inspecTimeEnd, 1);//加1天
                    ew.lt("inspec_time", inspecTimeEnd);
                }
            }

            if (object.has("inspecType")) {
                String inspecType = object.getString("inspecType");
                if (! StringUtils.isEmpty(inspecType)) {
                    ew.eq("inspec_type", inspecType.trim());
                }
            }

            if (object.has("status")) {
                String status = object.getString("status");
                if (! StringUtils.isEmpty(status)) {
                    ew.eq("status", Integer.parseInt(status.trim()));
                }
            }
        }
        ew.orderByDesc("inspec_time");
        Page page = new Page();
        if (pageIndex != MyConstant.ZERO && pageSize != MyConstant.ZERO) {
            page = new Page(pageIndex, pageSize);
        }
        Page<GrainSecurityInspect> pageResult = grainSecurityInspectMapper.selectPage(page, ew);
        return pageResult;
    }

    @Override
    public GrainSecurityInspectVO getDataById(int id) {
        GrainSecurityInspectVO vo = null;
        GrainSecurityInspect grainSecurityInspect = grainSecurityInspectMapper.selectById(id);
        if(null != grainSecurityInspect) {
            vo = new GrainSecurityInspectVO();
            BeanUtils.copyProperties(grainSecurityInspect, vo);
            //
            List<GrainSecurityInspectDetails> detailsList = grainSecurityInspectDetailsService.getByPid(id);
            vo.setDetailsList(detailsList);
        }
        return vo;
    }

    @Override
    @Transactional
    public void deleteById(int id) {
        grainSecurityInspectDetailsService.deleteByPid(id);
        grainSecurityInspectMapper.deleteById(id);
    }

    @Override
    @Transactional
    public void saveOrUpdateData(GrainSecurityInspectVO model, SysUserPO userPO) {
        GrainSecurityInspect grainSecurityInspect = null;
        if(null != model.getId()) {
            grainSecurityInspect = grainSecurityInspectMapper.selectById(model.getId());
        }

        if(null == grainSecurityInspect) {//不存在，即新增
            saveData(model, userPO);
        } else {//否则修改
            updateDate(model, grainSecurityInspect, userPO);
        }
    }

    private void updateDate(GrainSecurityInspectVO model, GrainSecurityInspect updateModel, SysUserPO userPO) {
        Date date = new Date();
        updateModel.setInspecType(model.getInspecType());
        updateModel.setInspecTime(date);
        updateModel.setStatus(model.getStatus());
        updateModel.setUpdateBy(userPO.getuId());
        updateModel.setUpdateTime(date);
        grainSecurityInspectMapper.updateById(updateModel);
        //
        List<GrainSecurityInspectDetails> detailsList = model.getDetailsList();
        if(null != detailsList && detailsList.isEmpty() == false) {
            for(GrainSecurityInspectDetails tempDetail : detailsList) {
                GrainSecurityInspectDetails detail = grainSecurityInspectDetailsService.getById(tempDetail.getId());
                if(tempDetail.getFlag() == null) {//没做选项，时间置空
                    detail.setDetailTime(null);
                } else if(detail.getFlag() == null || tempDetail.getFlag() != detail.getFlag()) {
                    detail.setDetailTime(date);
                }
                detail.setUpdateBy(userPO.getuId());
                detail.setUpdateTime(date);
                detail.setFlag(tempDetail.getFlag());
                grainSecurityInspectDetailsService.updateById(detail);
            }
        }
    }

    public void saveData(GrainSecurityInspectVO model, SysUserPO userPO) {
        Date date = new Date();
        model.setInspecTime(date);
        model.setCreateBy(userPO.getuId());
        model.setCreateTime(date);
        model.setUpdateBy(userPO.getuId());
        model.setUpdateTime(date);
        grainSecurityInspectMapper.insert(model);
        //
        List<GrainSecurityInspectDetails> detailsList = model.getDetailsList();
        if(null != detailsList && detailsList.isEmpty() == false) {
            for(GrainSecurityInspectDetails detail : detailsList) {
                detail.setPid(model.getId());
                if(null != detail.getFlag()) {
                    detail.setDetailTime(date);
                }
                detail.setCreateBy(userPO.getuId());
                detail.setCreateTime(date);
                detail.setUpdateBy(userPO.getuId());
                detail.setUpdateTime(date);
                grainSecurityInspectDetailsService.save(detail);
            }
        }
    }
}
