package com.unissoft.inspection.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.unissoft.MyConstant;
import com.unissoft.inspection.service.BranchDepotInspectDetailsService;
import com.unissoft.inspection.service.BranchDepotInspectService;
import com.unissoft.mapper.BranchDepotInspectMapper;
import com.unissoft.model.BranchDepotInspect;
import com.unissoft.model.BranchDepotInspectDetails;
import com.unissoft.model.SysUserPO;
import com.unissoft.result.PageParam;
import com.unissoft.utils.DateUtil;
import com.unissoft.vo.BranchDepotInspectVO;
import org.apache.commons.lang.StringUtils;
import org.apache.commons.lang.time.DateUtils;
import org.json.JSONObject;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.Date;
import java.util.List;

/**
 * <p>
 * 分库检查 服务实现类
 * </p>
 *
 * @author root
 * @since 2021-01-27
 */
@Service
public class BranchDepotInspectServiceImpl extends ServiceImpl<BranchDepotInspectMapper, BranchDepotInspect> implements BranchDepotInspectService {

    @Autowired
    private BranchDepotInspectMapper branchDepotInspectMapper;

    @Autowired
    private BranchDepotInspectDetailsService branchDepotInspectDetailsService;


    @Override
    public Page<BranchDepotInspect> getAllPage(Integer depotId, PageParam pageParam) {
        Integer pageIndex = pageParam.getPageIndex();
        Integer pageSize = pageParam.getPageSize();
        QueryWrapper<BranchDepotInspect> ew = new QueryWrapper<>();
        ew.ge("depot_id", depotId);
        if(! StringUtils.isEmpty(pageParam.getCondition())){
            JSONObject object = new JSONObject(pageParam.getCondition());

            if (object.has("inspecTimeStart")) {
                String inspecTimeStartStr = object.getString("inspecTimeStart");
                if (! StringUtils.isEmpty(inspecTimeStartStr)) {
                    Date inspecTimeStart = DateUtil.formatDate(inspecTimeStartStr, "yyyy-MM-dd");
                    ew.ge("inspec_time", inspecTimeStart);
                }
            }

            if (object.has("inspecTimeEnd")) {
                String inspecTimeEndStr = object.getString("inspecTimeEnd");
                if (! StringUtils.isEmpty(inspecTimeEndStr)) {
                    Date inspecTimeEnd = DateUtil.formatDate(inspecTimeEndStr, "yyyy-MM-dd");
                    inspecTimeEnd = DateUtils.addDays(inspecTimeEnd, 1);//加1天
                    ew.lt("inspec_time", inspecTimeEnd);
                }
            }

            if (object.has("status")) {
                String status = object.getString("status");
                if (! StringUtils.isEmpty(status)) {
                    ew.eq("status", Integer.parseInt(status.trim()));
                }
            }
        }
        ew.orderByDesc("inspec_time");
        Page page = new Page();
        if (pageIndex != MyConstant.ZERO && pageSize != MyConstant.ZERO) {
            page = new Page(pageIndex, pageSize);
        }
        Page<BranchDepotInspect> pageResult = branchDepotInspectMapper.selectPage(page, ew);
        return pageResult;
    }

    @Override
    public BranchDepotInspectVO getDataById(int id) {
        BranchDepotInspectVO vo = null;
        BranchDepotInspect branchDepotInspect = branchDepotInspectMapper.selectById(id);
        if(null != branchDepotInspect) {
            vo = new BranchDepotInspectVO();
            BeanUtils.copyProperties(branchDepotInspect, vo);
            //
            List<BranchDepotInspectDetails> detailsList = branchDepotInspectDetailsService.getByPid(id);
            vo.setDetailsList(detailsList);
        }
        return vo;
    }

    @Override
    @Transactional
    public void deleteById(int id) {
        branchDepotInspectDetailsService.deleteByPid(id);
        branchDepotInspectMapper.deleteById(id);
    }

    @Override
    @Transactional
    public void saveOrUpdateData(BranchDepotInspectVO model, SysUserPO userPO) {
        BranchDepotInspect branchDepotInspect = null;
        if(null != model.getId()) {
            branchDepotInspect = branchDepotInspectMapper.selectById(model.getId());
        }

        if(null == branchDepotInspect) {//不存在，即新增
            saveData(model, userPO);
        } else {//否则修改
            updateDate(model, branchDepotInspect, userPO);
        }
    }

    private void updateDate(BranchDepotInspectVO model, BranchDepotInspect updateModel, SysUserPO userPO) {
        Date date = new Date();
        updateModel.setInspecType(model.getInspecType());
        updateModel.setInspecTime(date);
        updateModel.setStatus(model.getStatus());
        updateModel.setUpdateBy(userPO.getuId());
        updateModel.setUpdateTime(date);
        branchDepotInspectMapper.updateById(updateModel);
        //
        List<BranchDepotInspectDetails> detailsList = model.getDetailsList();
        if(null != detailsList && detailsList.isEmpty() == false) {
            for(BranchDepotInspectDetails tempDetail : detailsList) {
                BranchDepotInspectDetails detail = branchDepotInspectDetailsService.getById(tempDetail.getId());
                if(tempDetail.getFlag() == null) {//没做选项，时间置空
                    detail.setDetailTime(null);
                } else if(detail.getFlag() == null || tempDetail.getFlag() != detail.getFlag()) {
                    detail.setDetailTime(date);
                }
                detail.setUpdateBy(userPO.getuId());
                detail.setUpdateTime(date);
                detail.setFlag(tempDetail.getFlag());
                branchDepotInspectDetailsService.updateById(detail);
            }
        }
    }

    public void saveData(BranchDepotInspectVO model, SysUserPO userPO) {
        Date date = new Date();
        model.setInspecTime(date);
        model.setCreateBy(userPO.getuId());
        model.setCreateTime(date);
        model.setUpdateBy(userPO.getuId());
        model.setUpdateTime(date);
        branchDepotInspectMapper.insert(model);
        //
        List<BranchDepotInspectDetails> detailsList = model.getDetailsList();
        if(null != detailsList && detailsList.isEmpty() == false) {
            for(BranchDepotInspectDetails detail : detailsList) {
                detail.setPid(model.getId());
                if(null != detail.getFlag()) {
                    detail.setDetailTime(date);
                }
                detail.setCreateBy(userPO.getuId());
                detail.setCreateTime(date);
                detail.setUpdateBy(userPO.getuId());
                detail.setUpdateTime(date);
                branchDepotInspectDetailsService.save(detail);
            }
        }
    }
}
