package com.unissoft.inspection.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.unissoft.MyConstant;
import com.unissoft.inspection.service.EntrustmentPointInspectDetailsService;
import com.unissoft.inspection.service.EntrustmentPointInspectService;
import com.unissoft.mapper.EntrustmentPointInspectMapper;
import com.unissoft.model.EntrustmentPointInspect;
import com.unissoft.model.EntrustmentPointInspectDetails;
import com.unissoft.model.SysUserPO;
import com.unissoft.result.PageParam;
import com.unissoft.utils.DateUtil;
import com.unissoft.vo.EntrustmentPointInspectVO;
import org.apache.commons.lang.StringUtils;
import org.apache.commons.lang.time.DateUtils;
import org.json.JSONObject;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.Date;
import java.util.List;

/**
 * <p>
 * 租仓、委托点检查 服务实现类
 * </p>
 *
 * @author root
 * @since 2021-01-27
 */
@Service
public class EntrustmentPointInspectServiceImpl extends ServiceImpl<EntrustmentPointInspectMapper, EntrustmentPointInspect> implements EntrustmentPointInspectService {

    @Autowired
    private EntrustmentPointInspectMapper entrustmentPointInspectMapper;

    @Autowired
    private EntrustmentPointInspectDetailsService entrustmentPointInspectDetailsService;

    @Override
    public Page<EntrustmentPointInspect> getAllPage(Integer depotId, PageParam pageParam) {
        Integer pageIndex = pageParam.getPageIndex();
        Integer pageSize = pageParam.getPageSize();
        QueryWrapper<EntrustmentPointInspect> ew = new QueryWrapper<>();
        ew.ge("depot_id", depotId);
        if(! StringUtils.isEmpty(pageParam.getCondition())){
            JSONObject object = new JSONObject(pageParam.getCondition());

            if (object.has("inspecTimeStart")) {
                String inspecTimeStartStr = object.getString("inspecTimeStart");
                if (! StringUtils.isEmpty(inspecTimeStartStr)) {
                    Date inspecTimeStart = DateUtil.formatDate(inspecTimeStartStr, "yyyy-MM-dd");
                    ew.ge("inspec_time", inspecTimeStart);
                }
            }

            if (object.has("inspecTimeEnd")) {
                String inspecTimeEndStr = object.getString("inspecTimeEnd");
                if (! StringUtils.isEmpty(inspecTimeEndStr)) {
                    Date inspecTimeEnd = DateUtil.formatDate(inspecTimeEndStr, "yyyy-MM-dd");
                    inspecTimeEnd = DateUtils.addDays(inspecTimeEnd, 1);//加1天
                    ew.lt("inspec_time", inspecTimeEnd);
                }
            }

            if (object.has("status")) {
                String status = object.getString("status");
                if (! StringUtils.isEmpty(status)) {
                    ew.eq("status", Integer.parseInt(status.trim()));
                }
            }
        }
        ew.orderByDesc("inspec_time");
        Page page = new Page();
        if (pageIndex != MyConstant.ZERO && pageSize != MyConstant.ZERO) {
            page = new Page(pageIndex, pageSize);
        }
        Page<EntrustmentPointInspect> pageResult = entrustmentPointInspectMapper.selectPage(page, ew);
        return pageResult;
    }

    @Override
    public EntrustmentPointInspectVO getDataById(int id) {
        EntrustmentPointInspectVO vo = null;
        EntrustmentPointInspect entrustmentPointInspect = entrustmentPointInspectMapper.selectById(id);
        if(null != entrustmentPointInspect) {
            vo = new EntrustmentPointInspectVO();
            BeanUtils.copyProperties(entrustmentPointInspect, vo);
            //
            List<EntrustmentPointInspectDetails> detailsList = entrustmentPointInspectDetailsService.getByPid(id);
            vo.setDetailsList(detailsList);
        }
        return vo;
    }

    @Override
    @Transactional
    public void deleteById(int id) {
        entrustmentPointInspectDetailsService.deleteByPid(id);
        entrustmentPointInspectMapper.deleteById(id);
    }

    @Override
    @Transactional
    public void saveOrUpdateData(EntrustmentPointInspectVO model, SysUserPO userPO) {
        EntrustmentPointInspect entrustmentPointInspect = null;
        if(null != model.getId()) {
            entrustmentPointInspect = entrustmentPointInspectMapper.selectById(model.getId());
        }

        if(null == entrustmentPointInspect) {//不存在，即新增
            saveData(model, userPO);
        } else {//否则修改
            updateDate(model, entrustmentPointInspect, userPO);
        }
    }

    private void updateDate(EntrustmentPointInspectVO model, EntrustmentPointInspect updateModel, SysUserPO userPO) {
        Date date = new Date();
        updateModel.setInspecType(model.getInspecType());
        updateModel.setInspecTime(date);
        updateModel.setStatus(model.getStatus());
        updateModel.setUpdateBy(userPO.getuId());
        updateModel.setUpdateTime(date);
        entrustmentPointInspectMapper.updateById(updateModel);
        //
        List<EntrustmentPointInspectDetails> detailsList = model.getDetailsList();
        if(null != detailsList && detailsList.isEmpty() == false) {
            for(EntrustmentPointInspectDetails tempDetail : detailsList) {
                EntrustmentPointInspectDetails detail = entrustmentPointInspectDetailsService.getById(tempDetail.getId());
                if(tempDetail.getFlag() == null) {//没做选项，时间置空
                    detail.setDetailTime(null);
                } else if(detail.getFlag() == null || tempDetail.getFlag() != detail.getFlag()) {
                    detail.setDetailTime(date);
                }
                detail.setUpdateBy(userPO.getuId());
                detail.setUpdateTime(date);
                detail.setFlag(tempDetail.getFlag());
                entrustmentPointInspectDetailsService.updateById(detail);
            }
        }
    }

    public void saveData(EntrustmentPointInspectVO model, SysUserPO userPO) {
        Date date = new Date();
        model.setInspecTime(date);
        model.setCreateBy(userPO.getuId());
        model.setCreateTime(date);
        model.setUpdateBy(userPO.getuId());
        model.setUpdateTime(date);
        entrustmentPointInspectMapper.insert(model);
        //
        List<EntrustmentPointInspectDetails> detailsList = model.getDetailsList();
        if(null != detailsList && detailsList.isEmpty() == false) {
            for(EntrustmentPointInspectDetails detail : detailsList) {
                detail.setPid(model.getId());
                if(null != detail.getFlag()) {
                    detail.setDetailTime(date);
                }
                detail.setCreateBy(userPO.getuId());
                detail.setCreateTime(date);
                detail.setUpdateBy(userPO.getuId());
                detail.setUpdateTime(date);
                entrustmentPointInspectDetailsService.save(detail);
            }
        }
    }
}
