package com.unissoft.inspection.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.unissoft.MyConstant;
import com.unissoft.inspection.service.LocalDepotInspectDetailsService;
import com.unissoft.inspection.service.LocalDepotInspectService;
import com.unissoft.mapper.LocalDepotInspectMapper;
import com.unissoft.model.LocalDepotInspect;
import com.unissoft.model.LocalDepotInspectDetails;
import com.unissoft.model.SysUserPO;
import com.unissoft.result.PageParam;
import com.unissoft.utils.DateUtil;
import com.unissoft.vo.LocalDepotInspectVO;
import org.apache.commons.lang.StringUtils;
import org.apache.commons.lang.time.DateUtils;
import org.json.JSONObject;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.*;

/**
 * <p>
 * 本库检查 服务实现类
 * </p>
 *
 * @author root
 * @since 2021-01-27
 */
@Service
public class LocalDepotInspectServiceImpl extends ServiceImpl<LocalDepotInspectMapper, LocalDepotInspect> implements LocalDepotInspectService {

    private static final String WEEK_INSPECT = "1";//分管科长每2周本库检查
    private static final String MONTH_INSPECT = "2";//分管副总每月本库检查

    @Autowired
    private LocalDepotInspectMapper localDepotInspectMapper;

    @Autowired
    private LocalDepotInspectDetailsService localDepotInspectDetailsService;

    @Override
    public List<LocalDepotInspect> getEveryTypeNewData(Integer depotId) {
        List<LocalDepotInspect> result = new ArrayList<>();
        //
        LocalDepotInspect dayInspect = getNewDataByType(depotId, WEEK_INSPECT);
        result.add(dayInspect);
        //
        LocalDepotInspect weekInspect = getNewDataByType(depotId, MONTH_INSPECT);
        result.add(weekInspect);
        //
        return result;
    }

    private LocalDepotInspect getNewDataByType(Integer depotId, String inspecType) {
        LocalDepotInspect result = null;
        Map<String, Object> param = new HashMap<>();
        param.put("depotId", depotId);
        param.put("inspecType", inspecType);
        List<LocalDepotInspect> dayInspectList = localDepotInspectMapper.getNewDataByType(param);
        if(null != dayInspectList && dayInspectList.isEmpty() == false) {
            result = dayInspectList.get(0);
        } else {
            result = new LocalDepotInspect();
            result.setInspecType(inspecType);
        }
        return result;
    }


    @Override
    public Page<LocalDepotInspect> getAllPage(Integer depotId, PageParam pageParam) {
        Integer pageIndex = pageParam.getPageIndex();
        Integer pageSize = pageParam.getPageSize();
        QueryWrapper<LocalDepotInspect> ew = new QueryWrapper<>();
        ew.ge("depot_id", depotId);
        if(! StringUtils.isEmpty(pageParam.getCondition())){
            JSONObject object = new JSONObject(pageParam.getCondition());

            if (object.has("inspecTimeStart")) {
                String inspecTimeStartStr = object.getString("inspecTimeStart");
                if (! StringUtils.isEmpty(inspecTimeStartStr)) {
                    Date inspecTimeStart = DateUtil.formatDate(inspecTimeStartStr, "yyyy-MM-dd");
                    ew.ge("inspec_time", inspecTimeStart);
                }
            }

            if (object.has("inspecTimeEnd")) {
                String inspecTimeEndStr = object.getString("inspecTimeEnd");
                if (! StringUtils.isEmpty(inspecTimeEndStr)) {
                    Date inspecTimeEnd = DateUtil.formatDate(inspecTimeEndStr, "yyyy-MM-dd");
                    inspecTimeEnd = DateUtils.addDays(inspecTimeEnd, 1);//加1天
                    ew.lt("inspec_time", inspecTimeEnd);
                }
            }

            if (object.has("inspecType")) {
                String inspecType = object.getString("inspecType");
                if (! StringUtils.isEmpty(inspecType)) {
                    ew.eq("inspec_type", inspecType.trim());
                }
            }

            if (object.has("status")) {
                String status = object.getString("status");
                if (! StringUtils.isEmpty(status)) {
                    ew.eq("status", Integer.parseInt(status.trim()));
                }
            }
        }
        ew.orderByDesc("inspec_time");
        Page page = new Page();
        if (pageIndex != MyConstant.ZERO && pageSize != MyConstant.ZERO) {
            page = new Page(pageIndex, pageSize);
        }
        Page<LocalDepotInspect> pageResult = localDepotInspectMapper.selectPage(page, ew);
        return pageResult;
    }

    @Override
    public LocalDepotInspectVO getDataById(int id) {
        LocalDepotInspectVO vo = null;
        LocalDepotInspect localDepotInspect = localDepotInspectMapper.selectById(id);
        if(null != localDepotInspect) {
            vo = new LocalDepotInspectVO();
            BeanUtils.copyProperties(localDepotInspect, vo);
            //
            List<LocalDepotInspectDetails> detailsList = localDepotInspectDetailsService.getByPid(id);
            vo.setDetailsList(detailsList);
        }
        return vo;
    }

    @Override
    @Transactional
    public void deleteById(int id) {
        localDepotInspectDetailsService.deleteByPid(id);
        localDepotInspectMapper.deleteById(id);
    }

    @Override
    @Transactional
    public void saveOrUpdateData(LocalDepotInspectVO model, SysUserPO userPO) {
        LocalDepotInspect localDepotInspect = null;
        if(null != model.getId()) {
            localDepotInspect = localDepotInspectMapper.selectById(model.getId());
        }

        if(null == localDepotInspect) {//不存在，即新增
            saveData(model, userPO);
        } else {//否则修改
            updateDate(model, localDepotInspect, userPO);
        }
    }

    private void updateDate(LocalDepotInspectVO model, LocalDepotInspect updateModel, SysUserPO userPO) {
        Date date = new Date();
        updateModel.setInspecType(model.getInspecType());
        updateModel.setInspecTime(date);
        updateModel.setStatus(model.getStatus());
        updateModel.setUpdateBy(userPO.getuId());
        updateModel.setUpdateTime(date);
        localDepotInspectMapper.updateById(updateModel);
        //
        List<LocalDepotInspectDetails> detailsList = model.getDetailsList();
        if(null != detailsList && detailsList.isEmpty() == false) {
            for(LocalDepotInspectDetails tempDetail : detailsList) {
                LocalDepotInspectDetails detail = localDepotInspectDetailsService.getById(tempDetail.getId());
                if(tempDetail.getFlag() == null) {//没做选项，时间置空
                    detail.setDetailTime(null);
                } else if(detail.getFlag() == null || tempDetail.getFlag() != detail.getFlag()) {
                    detail.setDetailTime(date);
                }
                detail.setUpdateBy(userPO.getuId());
                detail.setUpdateTime(date);
                detail.setFlag(tempDetail.getFlag());
                localDepotInspectDetailsService.updateById(detail);
            }
        }
    }

    public void saveData(LocalDepotInspectVO model, SysUserPO userPO) {
        Date date = new Date();
        model.setInspecTime(date);
        model.setCreateBy(userPO.getuId());
        model.setCreateTime(date);
        model.setUpdateBy(userPO.getuId());
        model.setUpdateTime(date);
        localDepotInspectMapper.insert(model);
        //
        List<LocalDepotInspectDetails> detailsList = model.getDetailsList();
        if(null != detailsList && detailsList.isEmpty() == false) {
            for(LocalDepotInspectDetails detail : detailsList) {
                detail.setPid(model.getId());
                if(null != detail.getFlag()) {
                    detail.setDetailTime(date);
                }
                detail.setCreateBy(userPO.getuId());
                detail.setCreateTime(date);
                detail.setUpdateBy(userPO.getuId());
                detail.setUpdateTime(date);
                localDepotInspectDetailsService.save(detail);
            }
        }
    }
}
