package com.unissoft.policySystem.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.unissoft.MyConstant;
import com.unissoft.model.EquipmentStroe;
import com.unissoft.model.PolicySystem;
import com.unissoft.mapper.PolicySystemMapper;
import com.unissoft.policySystem.service.PolicySystemService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.unissoft.policySystem.utils.minio.MinioUtils;
import com.unissoft.result.PageParam;
import org.json.JSONObject;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.StringUtils;
import org.springframework.web.multipart.MultipartFile;

/**
 * <p>
 *  服务实现类
 * </p>
 *
 * @author root
 * @since 2020-12-15
 */
@Service
public class PolicySystemServiceImpl extends ServiceImpl<PolicySystemMapper, PolicySystem> implements PolicySystemService {

    @Autowired
    private  PolicySystemMapper  policySystemMapper;

    @Autowired
    private MinioUtils minioUtils;

    @Override
    public Page<PolicySystem> getPage(PageParam pageParam) {
        int pageIndex = pageParam.getPageIndex();
        int pageSize = pageParam.getPageSize();
        QueryWrapper<PolicySystem> ew = new QueryWrapper<>();
        if (!StringUtils.isEmpty(pageParam.getCondition())) {
            JSONObject object = new JSONObject(pageParam.getCondition());
            String assetNumber = object.optString("asset_number").trim();
            String equipmentCategory = object.optString("equipment_category").trim();
            String equipmentName = object.optString("equipment_name").trim();
            String standardsType = object.optString("standards_type").trim();
            String arrivalTime = object.optString("arrival_time").trim();
            if (!StringUtils.isEmpty(assetNumber)) {
                ew.eq("asset_number", assetNumber);
            }
            if (!StringUtils.isEmpty(equipmentCategory)) {
                ew.eq("equipment_category", equipmentCategory);
            }
            if (!StringUtils.isEmpty(equipmentName)) {
                ew.eq("equipment_name", equipmentName);
            }
            if (!StringUtils.isEmpty(standardsType)) {
                ew.eq("standards_type", standardsType);
            }
        }
        ew.orderByDesc("arrivalTime");
        Page page = new Page();
        if (pageIndex != MyConstant.ZERO && pageSize != MyConstant.ZERO) {
            page = new Page(pageIndex, pageSize);
        }
        Page<PolicySystem> pageResult = policySystemMapper.selectPage(page, ew);
        return pageResult;
    }


    @Override
    public void deleteById(Integer id) {
        policySystemMapper.deleteById(id);
    }

    @Override
    public void savePolicySystem(PolicySystem model) {
        policySystemMapper.insert(model);
    }

    @Override
    public String savePolicySystemAttachment(MultipartFile file, Integer id) throws Exception {
        return uploadFile( file, id);
    }


    @Transactional
    public String uploadFile(MultipartFile file, Integer userId) throws Exception {
        String str = "";
        if (file != null && userId != null) {
            //1.
            PolicySystem policySystem = policySystemMapper.selectById(userId);
            if (policySystem == null) {
                return str;
            }
            if (org.apache.commons.lang.StringUtils.isNotBlank(policySystem.getAttachmentPath())) {
                //2.如果存在附件先删除在上传
                minioUtils.removeObject(MyConstant.ATTACHMENT_PATH, policySystem.getAttachmentPath());
            }
            //上传图片返回文件名称
            str = minioUtils.uploadFile(file, MyConstant.ATTACHMENT_PATH);
            policySystem.setId(userId);
            policySystem.setAttachmentPath(str);
            policySystemMapper.updateById(policySystem);
        }
        return str;
    }
}
