package com.unissoft.quantityQuality.controller;


import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.unissoft.model.QuantityQuality;
import com.unissoft.quantityQuality.service.QuantityQualityService;
import com.unissoft.result.PageParam;
import com.unissoft.result.ResultView;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * <p>
 * 前端控制器
 * </p>
 *
 * @author root
 * @since 2021-01-25
 */
@Api(description = "库存出库质量控制类")
@RestController
@RequestMapping("/quantityQuality")
public class QuantityQualityController {

    @Autowired
    private QuantityQualityService qualityService;

    @ApiOperation(value = "分页条件查询", notes = "depotYpte=0库存质量，1出库质量；depotId库点编码，warehouseId仓房编码，qType检查类别 。depotYpte(必填)  {\n" +
            "  \"condition\": \"{'depotYpte':'1','depotId':'','warehouseId':'','statTime':'','endTime':'','qType':''}\",\n" +
            "  \"pageIndex\": 1,\n" +
            "  \"pageSize\": 10\n" +
            "}")
    @PostMapping("/authority/getPage")
    public ResultView getPage(@RequestBody PageParam pageParam) {
        Page<QuantityQuality> page = qualityService.getPage(pageParam);
        return ResultView.success(page);
    }

    @ApiOperation(value = "查询最新的库存质量，出库质量depotYpte(必填)=0库存质量，1出库质量", notes = "")
    @GetMapping("/authority/groudByType")
    public ResultView groudByType(@RequestParam String depotYpte, @RequestParam(required = false) String depotId) {
        List<QuantityQuality> quantityQualities = qualityService.groudByType(depotId, depotYpte);
        return ResultView.success(quantityQualities);
    }

    @ApiOperation(value = "根据id查询", notes = "")
    @GetMapping("/authority/getById")
    public ResultView getById(@RequestParam int id) {
        QuantityQuality model = qualityService.getById(id);
        return ResultView.success(model);
    }

    @ApiOperation(value = "新增", notes = "")
    @PostMapping("/authority_button/add")
    public ResultView add(@RequestBody QuantityQuality model) {
        qualityService.saveRole(model);
        return ResultView.success(model);
    }

    @ApiOperation(value = "修改", notes = "")
    @PostMapping("/authority_button/update")
    public ResultView update(@RequestBody QuantityQuality model) {
        qualityService.updateRole(model);
        return ResultView.success(model);
    }

    @ApiOperation(value = "根据id删除", notes = "")
    @DeleteMapping("/authority_button/deleteById")
    public ResultView deleteById(@RequestParam Integer id) {
        qualityService.deleteById(id);
        return ResultView.success();
    }

}
