package com.unissoft.quantityQuality.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.unissoft.MyConstant;
import com.unissoft.mapper.QuantityQualityMapper;
import com.unissoft.model.QuantityQuality;
import com.unissoft.quantityQuality.service.QuantityQualityService;
import com.unissoft.result.PageParam;
import org.json.JSONObject;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import java.util.Date;
import java.util.List;

/**
 * <p>
 * 库存出库质量表 服务实现类
 * </p>
 *
 * @author root
 * @since 2021-01-26
 */
@Service
public class QuantityQualityServiceImpl extends ServiceImpl<QuantityQualityMapper, QuantityQuality> implements QuantityQualityService {

    @Autowired
    private QuantityQualityMapper qualityMapper;

    @Override
    public Page<QuantityQuality> getPage(PageParam pageParam) {
        int pageIndex = pageParam.getPageIndex();
        int pageSize = pageParam.getPageSize();
        QueryWrapper<QuantityQuality> ew = new QueryWrapper<>();
        String depotId = "", depotYpte = "";
        if (!StringUtils.isEmpty(pageParam.getCondition())) {
            JSONObject object = new JSONObject(pageParam.getCondition());
            depotYpte = object.optString("depotYpte");
            depotId = object.optString("depotId");
            String warehouseId = object.optString("warehouseId");
            String statTime = object.optString("statTime");
            String endTime = object.optString("endTime");
            String qType = object.optString("qType");

            if (!StringUtils.isEmpty(depotYpte)) {
                ew.eq("depot_ypte", depotYpte);
            }
            if (!StringUtils.isEmpty(depotId)) {
                ew.eq("depot_id", depotId);
            }
            if (!StringUtils.isEmpty(warehouseId)) {
                ew.eq("warehouse_id", warehouseId);
            }
            if (!StringUtils.isEmpty(statTime) && !StringUtils.isEmpty(endTime)) {
                statTime += MyConstant.STR_TIME;
                endTime += MyConstant.STR_END;
                ew.between("create_time", statTime, endTime);
            }
            if (!StringUtils.isEmpty(qType)) {
                ew.eq("q_type", qType);
            }
        }
        ew.isNotNull("depot_id");
        ew.orderByDesc("create_time");
        Page page = new Page();
        if (pageIndex != MyConstant.ZERO && pageSize != MyConstant.ZERO) {
            page = new Page(pageIndex, pageSize);
        }

        Page<QuantityQuality> pageResult = qualityMapper.selectPage(page, ew);

        return pageResult;
    }

    @Override
    public List<QuantityQuality> groudByType(String depotId, String depotYpte) {
        List<QuantityQuality> quantityQualities = qualityMapper.groudByType(depotId, depotYpte);
        return quantityQualities;
    }

    @Override
    public void saveRole(QuantityQuality model) {
        Date date = new Date();
        model.setCreateTime(date);
        qualityMapper.insert(model);
    }

    @Override
    public void updateRole(QuantityQuality model) {
        Date date = new Date();
        model.setCreateTime(date);
        qualityMapper.updateById(model);
    }

    @Override
    public void deleteById(Integer id) {
        QuantityQuality quality = qualityMapper.selectById(id);
        if (quality != null && quality.getDepotId() != null) {
            qualityMapper.deleteById(id);
        }
    }
}
