package com.unissoft.quantityQuality.service.impl;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.unissoft.MyConstant;
import com.unissoft.mapper.WarehousingQualityMapper;
import com.unissoft.model.WarehousingQuality;
import com.unissoft.model.WarehousingQualityPO;
import com.unissoft.model.WarehousingQualityVO;
import com.unissoft.quantityQuality.service.WarehousingQualityChildrenService;
import com.unissoft.quantityQuality.service.WarehousingQualityService;
import com.unissoft.result.PageParam;
import org.apache.commons.lang.StringUtils;
import org.json.JSONObject;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.Date;
import java.util.List;

/**
 * <p>
 * 入库质量表名称 服务实现类
 * </p>
 *
 * @author root
 * @since 2021-04-12
 */
@Service
public class WarehousingQualityServiceImpl extends ServiceImpl<WarehousingQualityMapper, WarehousingQuality> implements WarehousingQualityService {

    @Autowired
    private WarehousingQualityMapper qualityMapper;

    @Autowired
    private WarehousingQualityChildrenService qualityChildrenService;

    @Override
    public IPage<WarehousingQualityPO> getPage(PageParam pageParam) {
        Integer pageIndex = pageParam.getPageIndex();
        Integer pageSize = pageParam.getPageSize();
        Page<WarehousingQualityPO> page = null;
        if (pageIndex != MyConstant.ZERO && pageSize != MyConstant.ZERO) {
            page = new Page<WarehousingQualityPO>(pageIndex, pageSize);
        }
        String depotId = null, warehouseId = null, creationTimeStart = null, creationTimeEnd = null;
        if (StringUtils.isNotBlank(pageParam.getCondition())) {
            JSONObject object = new JSONObject(pageParam.getCondition());
            depotId = object.optString("depotId");
            warehouseId = object.optString("warehouseId");
            creationTimeStart = object.optString("creationTimeStart").trim();
            creationTimeEnd = object.optString("creationTimeEnd").trim();
        }

        if (!StringUtils.isEmpty(creationTimeStart) && !StringUtils.isEmpty(creationTimeEnd)) {
            creationTimeStart += MyConstant.STR_TIME_TWO;
            creationTimeEnd += MyConstant.STR_TIME_TWO;
        }
        IPage<WarehousingQualityPO> page1 = qualityMapper.getPage(page, depotId, warehouseId, creationTimeStart, creationTimeEnd);
        return page1;
    }

    @Override
    public WarehousingQualityPO getByIdQality(int id) {
        List<WarehousingQualityPO> byIdQality = qualityMapper.getByIdQality(id);
        WarehousingQualityPO warehousingQualityPO = null;
        if (byIdQality.size() > MyConstant.ZERO) {
            warehousingQualityPO = byIdQality.get(MyConstant.ZERO);
        }
        return warehousingQualityPO;
    }

    @Transactional
    @Override
    public void saveWarehousingQuality(WarehousingQualityVO model) {
        Date date = new Date();
        WarehousingQuality quality = null;
        //标识是否第一次新增，0代表第一次，1代表n次
        if (model.getSaveType() == MyConstant.ZERO) {
            quality = new WarehousingQuality(model.getDepotId(), model.getDepotName(), model.getWarehouseId(), model.getWarehouseName(), model.getCheckName(), model.getCheckType(), date, null, MyConstant.ZERO);
            qualityMapper.insert(quality);
        } else {
            Date endTime = null;
            Integer qualityState = null;
            //检测类型（0：1/3,1:1/2,2:2/3,3:自验收,4:质检中心验收,5:质检中心巡检）
            if (model.getCheckType() == MyConstant.FOUR) {
                endTime = new Date();
                qualityState = MyConstant.ONE;
            }
            quality = new WarehousingQuality(model.getDepotId(), model.getDepotName(), model.getWarehouseId(), model.getWarehouseName(), model.getCheckName(), model.getCheckType(), null, endTime, qualityState);
            quality.setId(model.getId());
            qualityMapper.updateById(quality);
        }
        //操作字表
        model.setCreationTime(date);
        model.setId(quality.getId());
        qualityChildrenService.saveWarehousingQualityChildren(model);
    }

    @Transactional
    @Override
    public void updateWarehousingQuality(WarehousingQualityVO model) {
        WarehousingQuality quality = new WarehousingQuality(model.getDepotId(), model.getDepotName(), model.getWarehouseId(), model.getWarehouseName(), model.getCheckName(), model.getCheckType(), null, null, null);
        quality.setId(model.getId());
        qualityMapper.updateById(quality);
        qualityChildrenService.updateWarehousingQualityChildren(model);
    }

    @Transactional
    @Override
    public void deleteById(int id) {
        qualityMapper.deleteById(id);
        qualityChildrenService.deleteByQualityId(id);
    }
}
