package com.unissoft.scheduled;

import com.alibaba.fastjson.JSONArray;
import com.google.common.collect.Lists;
import com.unissoft.MyConstant;
import com.unissoft.dto.UserVO;
import com.unissoft.feign.SystemFeign;
import com.unissoft.model.ProcessAudit;
import com.unissoft.model.StatisticsWorkCompletion;
import com.unissoft.model.SysUserActivity;
import com.unissoft.model.SysUserDTO;
import com.unissoft.utils.DateUtil;
import com.unissoft.utils.RedisUtil;
import com.unissoft.ventilation.service.ProcessAuditService;
import com.unissoft.ventilation.service.SysUserActivityService;
import com.unissoft.ventilation.service.impl.StatisticsWorkCompletionServiceImpl;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import java.text.ParseException;
import java.util.*;
import java.util.stream.Collectors;

/**
 * 文件描述:首页右侧定时统计工作完成情况
 *
 * @author tongxi.xia
 * @date 2021年02月26日 9:47
 */
@Component
public class TimingStatistics {

    @Autowired
    private StatisticsWorkCompletionServiceImpl statisticsWorkCompletionService;

    @Autowired
    private ProcessAuditService processAuditService;

    @Autowired
    private SysUserActivityService userActivityService;

    @Autowired
    private SystemFeign systemFeign;

    @Autowired
    private RedisUtil redisUtil;

    /**
     * 每月統計工作完成情況,每个月最后一天晚上23:50分执行
     *
     * @throws ParseException
     */
//    @Scheduled(cron = "0 */2 * * * ?")
    @Transactional
    @Scheduled(cron = "0 30 23 28-31 * ?")
    public void saveSysLog() throws ParseException {
        final Calendar calendar = Calendar.getInstance();
        //如果不是最后一天
        if (!(calendar.get(Calendar.DATE) == calendar.getActualMaximum(Calendar.DATE))) {
            return;
        }
        List<ProcessAudit> currentMonthData = processAuditService.getCurrentMonthData();

        if (currentMonthData != null && currentMonthData.size() > MyConstant.ZERO) {
            HashMap<Integer, SysUserDTO> byUserAll = systemFeign.getByUserAll();
            ArrayList<StatisticsWorkCompletion> workCompletions = Lists.newArrayList();
            //根据人员分组
            Map<Integer, List<ProcessAudit>> collect = currentMonthData.stream().collect(Collectors.groupingBy(t -> t.getToPeople()));
            int finishOnTime = MyConstant.ZERO;
            int sScore = MyConstant.ONE_HUNDRED;
            for (Integer key : collect.keySet()) {
                //获取人员和库信息
                SysUserDTO userById = byUserAll.get(key);
                List<ProcessAudit> processAudits = collect.get(key);
                //审批状态(0:提交,1审批通过:,2:审批不通过，3:审批完成进行中，4已完成
                Map<Integer, List<ProcessAudit>> collect1 = processAudits.stream().collect(Collectors.groupingBy(t -> t.getAuditState()));
                List<ProcessAudit> processAudits1 = collect1.get(MyConstant.FOUR);
                if (processAudits1 != null && processAudits1.size() > MyConstant.ZERO) {
                    //待办完成
                    finishOnTime = processAudits1.size();
                    //肯定是 100➗N   n 是待办数    然后拿着得到的值 ✖️已完成未逾期的 = 月底积分
                    sScore = (MyConstant.ONE_HUNDRED / processAudits.size() * processAudits1.size());
                } else {
                    finishOnTime = MyConstant.ZERO;
                    sScore = MyConstant.ONE_HUNDRED;
                }
                /*当前时间大于待办时间and审批状态不等于4，待办逾期；
                当前时间大于待办时间and审批状态等于4，待办逾期完成；*/
                //int compareTo = beginTime.compareTo(endTime); compareTo()方法的返回值，date1小于date2返回-1，date1大于date2返回1，相等返回0
                //获取当前时间
                Date time = DateUtil.getTime();
                int complete = processAudits.stream().filter(t -> t.getAuditState() == MyConstant.FOUR && time.compareTo(t.getEndDate()) == MyConstant.ONE).collect(Collectors.toList()).size();
                int incomplete = processAudits.stream().filter(t -> t.getAuditState() != MyConstant.FOUR && time.compareTo(t.getEndDate()) == MyConstant.ONE).collect(Collectors.toList()).size();
                StatisticsWorkCompletion workCompletion = new StatisticsWorkCompletion(key, userById.getPersonnelName(), userById.getPPost(), userById.getDepotId(), userById.getDepotName(), processAudits.size(), finishOnTime, complete, incomplete, sScore, time);
                workCompletions.add(workCompletion);
            }
            statisticsWorkCompletionService.saveBatch(workCompletions);

           /* 基础分60。一天不登录扣0.5分；
            登录但不处理业务不得分也不扣分；
            登录并处理业务得0.5分。系统分值存储没有上下限，但显示区间为0~100，低于0的显示为0，高于100显示为100。
            （若分数为负，比如-3，此时显示为0分，处理业务加0.5后为-2.5，仍显示为0，这种情况是允许的*/
            ArrayList<SysUserActivity> userActivitys = Lists.newArrayList();
            //获取所有用户
            Map<String, String> userAll = (Map<String, String>) systemFeign.getAll(null).getData();
            //获取缓存中登录用户
            //获取当前时间年月日
            Date specificTime1 = DateUtil.getSpecificTime();
            //获取当前月的天数
            int currentMonthDay = DateUtil.getCurrentMonthDay();

            List<Object> objects = redisUtil.lGet(MyConstant.SPECIFIC_TIME, MyConstant.ZERO, MyConstant.NEGATIVE_ONE);
            List<UserVO> userVOS = JSONArray.parseArray(objects.toString(), UserVO.class);
            //去重
            ArrayList<UserVO> collect1 = userVOS.stream().collect(Collectors.collectingAndThen(Collectors.toCollection(() -> new TreeSet<>(Comparator.comparing(o -> o.getId() + ":" + o.getTime()))), ArrayList::new));
            //根据用户id统计登录的天数
            Map<Integer, Long> collect2 = collect1.stream().collect(Collectors.groupingBy(o -> o.getId(), Collectors.counting()));
            for (Integer key : collect2.keySet()) {
                int i = collect2.get(key).intValue();
                //获取单个用户的登录得分
                Double i1 = MyConstant.SIXTY - ((currentMonthDay - i) * MyConstant.ZERO_POINT_FIVE);
                int activity = i1.intValue();
                StatisticsWorkCompletion workCompletion = workCompletions.stream().filter(o -> o.getPersonnelId().equals(key)).findAny().orElse(null);
                if (workCompletion != null) {
                    Integer finishOnTime1 = workCompletion.getFinishOnTime();
                    if (finishOnTime1 != null && finishOnTime1 != MyConstant.ZERO) {
                        Double v = i1 + (finishOnTime1 * MyConstant.ZERO_POINT_FIVE);
                        activity = v.intValue();
                    }
                }
                SysUserDTO userById = byUserAll.get(key);
                userActivitys.add(new SysUserActivity(key, userById.getPersonnelName(), userById.getDepotId(), userById.getDepotName(), userById.getPPost(), specificTime1, activity));
            }

            List<String> list1 = new ArrayList<String>(userAll.keySet());
            List<Integer> list2 = new ArrayList<Integer>(collect2.keySet());
            // 差集 (list2 - list1)
            List<Integer> collectf = Lists.newArrayList();
            List<Integer> collect3 = Lists.newArrayList();
            for (int i = 0; i < list2.size(); i++) {
                Integer integer = list2.get(i);
                if (!collectf.contains(integer)) {
                    collectf.add(integer);
                }
            }
            for (int j = 0; j < list1.size(); j++) {
                String s = list1.get(j);
                Integer integer1 = Integer.valueOf(s);
                if (!collectf.contains(integer1)) {
                    collect3.add(integer1);
                }
            }

            for (Integer userId : collect3) {
                SysUserDTO userById = byUserAll.get(userId);
                Double act = MyConstant.SIXTY - (currentMonthDay * MyConstant.ZERO_POINT_FIVE);
                int activity = act.intValue();
                userActivitys.add(new SysUserActivity(userId, userById.getPersonnelName(), userById.getDepotId(), userById.getDepotName(), userById.getPPost(), specificTime1, activity));
            }
            //降序排序
            List<SysUserActivity> collect4 = userActivitys.stream().sorted(Comparator.comparing(SysUserActivity::getUsActivity).reversed()).collect(Collectors.toList());
            int i = 0;
            for (SysUserActivity act : collect4) {
                act.setProvinceRanking(++i);
            }
            //根据库分组
            Map<Integer, List<SysUserActivity>> depotMap = userActivitys.stream().collect(Collectors.groupingBy(SysUserActivity::getDepotId));
            for (Integer depotId : depotMap.keySet()) {
                i = 0;
                List<SysUserActivity> sysUserActivities = depotMap.get(depotId);
                List<SysUserActivity> sctivitys = sysUserActivities.stream().sorted(Comparator.comparing(SysUserActivity::getUsActivity).reversed()).collect(Collectors.toList());
                for (SysUserActivity act : sctivitys) {
                    act.setDepotRanking(++i);
                }
            }
            userActivityService.saveBatch(userActivitys);
            redisUtil.del(MyConstant.SPECIFIC_TIME);
        }
    }
}
