package com.unissoft.ventilation.controller;


import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.unissoft.dto.ProcessToBeDone;
import com.unissoft.dto.ToBeDoneCallbackVO;
import com.unissoft.feign.SystemFeign;
import com.unissoft.model.ProcessAudit;
import com.unissoft.model.SysUserDTO;
import com.unissoft.result.PageParam;
import com.unissoft.result.ResultView;
import com.unissoft.ventilation.service.ProcessAuditService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.json.JSONException;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;
import java.util.Map;

/**
 * <p>
 *  前端控制器
 * </p>
 *
 * @author My SunShine
 * @since 2020-12-08
 */
@Api(description = "审批流程详情控制器")
@RestController
@RequestMapping("/ventilation/processAudit")
public class ProcessAuditController {

	@Autowired
    private ProcessAuditService processAuditService;
	
	/**
	 * 新增一个审批流程数据第一次 新增提交
	 * @param parameter
	 * @return
	 */
	@ApiOperation(value = "新增", notes = "")
    @PostMapping("/add")
    public ResultView add(@RequestBody String parameter) throws JSONException{
		processAuditService.add(parameter);
        return ResultView.success();
    }
	
	
	/**
	 * 机械通风列表
	 */
    @ApiOperation(value = "分页条件查询", notes = "提交参数(type、houseId、depotId必填,没有houseId的就不填)：{\"pageIndex\":1,\"pageSize\":10,  \"condition\":\"{\'equipmentId\':\'\',\'type\':\'\',\'depotId\':\'\',\'houseId\':\'\',\'startTime\':\'\',\'endTime\':\'\'}\"}")
    @PostMapping("/getPage")
    public ResultView getPage(@RequestBody PageParam pageParam) throws Exception{
    	Page<ProcessAudit> page = processAuditService.getPage(pageParam);
        return ResultView.success(page);
    }
    
    /**
     * 通过审批第一条数据的id查询所有数据
     * @param id
     * @return
     */
    /*@ApiOperation(value = "根据id和审批状态查询", notes = "")
    @GetMapping("/getById/{id}")
    public ResultView getById(@PathVariable Integer id) {
    	List<Map<String,Object>> list = processAuditService.getAllById(id);
        return ResultView.success(list);
    }*/
    
    
    /**
     * 删除
     * @param id
     * @return
     */
    @ApiOperation(value = "根据id删除数据", notes = "")
    @GetMapping("/deleteById/{id}")
    public ResultView deleteById(@PathVariable Integer id) {
    	 processAuditService.deleteById(id);
        return ResultView.success();
    }
    
    
    /**
     * 待审批列表(待处理列表)
     * @throws Exception 
     */
    @ApiOperation(value = "分页条件查询,userId必填", notes = "提交参数：{\"pageIndex\":1,\"pageSize\":10,\"condition\":\"{\'userId\':\'\'}\"}")
    @PostMapping("/toDisposeList")
    public ResultView toDisposeList(@RequestBody PageParam pageParam) throws Exception {
        Page<ProcessAudit> page = processAuditService.toDisposeList(pageParam);
        return ResultView.success(page);
    }
    
    /**
     * 通风审批
     */
    @ApiOperation(value = "审批", notes = "")
    @PostMapping("/audit")
    public ResultView audit(@RequestBody String parameter) {
		processAuditService.audit(parameter);
        return ResultView.success();
    }
    
    /**
     * 药剂采购
     * @param parameter
     * @return
     */
    @ApiOperation(value = "药剂审批", notes = "")
    @PostMapping("/yjcgAudit")
    public ResultView yjcgAudit(@RequestBody String parameter) {
		processAuditService.yjcgAudit(parameter);
        return ResultView.success();
    }
    
    
    /**
     * 通风记录详情的增加(审批之后的操作)
     */
    @ApiOperation(value = "通风记录详情的增加", notes = "")
    @PostMapping("/detailsAdd")
    public ResultView detailsAdd(@RequestBody String parameter) throws JSONException{
		processAuditService.detailsAdd(parameter);
        return ResultView.success();
    }
    
    /**
     * 根据通风的流程id 查询业务详情
     * @param
     * @return
     */
    @ApiOperation(value = "根据id查询数据", notes = "")
    @GetMapping("/getDataByProcessId/{processId}")
    public ResultView getDataByProcessId(@PathVariable String processId) {
    	Map<String,Object> map = processAuditService.getDataByProcessId(processId);
        return ResultView.success(map);
    }
    
    /**
     * 待办列表(已发起)
     * 申请人是当前登录人
     * @throws Exception 
     */
    @ApiOperation(value = "分页条件查询,userId必填", notes = "提交参数：{\"pageIndex\":1,\"pageSize\":10,\"condition\":\"{\'userId\':\'\'}\"}")
    @PostMapping("/applyList")
    public ResultView applyList(@RequestBody PageParam pageParam) throws Exception {
        Page<ProcessAudit> page = processAuditService.applyList(pageParam);
        return ResultView.success(page);
    }
    
    
    /**
     * 待办列表(已处理)
     * 申请人是当前登录人
     * @throws Exception 
     */
    @ApiOperation(value = "分页条件查询,userId必填", notes = "提交参数：{\"pageIndex\":1,\"pageSize\":10,\"condition\":\"{\'userId\':\'\'}\"}")
    @PostMapping("/completeList")
    public ResultView completeList(@RequestBody PageParam pageParam) throws Exception {
        Page<ProcessAudit> page = processAuditService.completeList(pageParam);
        return ResultView.success(page);
    }
    
    /**
     * 粮情生成待办事项
     */
    @ApiOperation(value = "粮情生成待办事项", notes = "")
    @PostMapping("/toBeDone")
    public List<ToBeDoneCallbackVO> toBeDone(@RequestBody ProcessToBeDone processToBeDone) {
        List<ToBeDoneCallbackVO> result = processAuditService.toBeDone(processToBeDone);
        return result;
    }
    
    
    /**
     * 根据待办id，将流程设成已办
       同时要存储具体办理时的时间，因为要计算该流程办理的周期(或计算逾期时间)
     * @param id
     */
    @ApiOperation(value = "根据待办id，将流程设成已办", notes = "")
    @GetMapping("/haveDone/{id}/{auditState}")
    public ResultView haveDone(@PathVariable Integer id,@PathVariable Integer auditState) {
    	 processAuditService.haveDone(id, auditState);
    	 return ResultView.success();
    }
    
    
    /**
     * 根据用户id、流程类型(如科长每2周粮情检查的类型：xxxx1)，按时间倒序查出待办流程，将最近日期的流程设成已办
       同时要存储具体办理时的时间，因为要计算该流程办理的周期(或计算逾期时间)
     */
    @ApiOperation(value = "根据待办id，将流程设成已办", notes = "")
    @GetMapping("/haveDone/{userId}/{type}/{depotId}/{houseId}/{auditState}")
    public ResultView haveDone(@PathVariable Integer userId, @PathVariable String type, @PathVariable Integer depotId, @PathVariable Integer houseId,@PathVariable Integer auditState) {
    	 processAuditService.haveDone(userId,type,depotId,houseId,auditState);
    	 return ResultView.success();
    }

    @ApiOperation(value = "根据待办条件：用户ID、类型、库ID，将流程设成已办", notes = "")
    @GetMapping("/haveDone2/{userId}/{type}/{depotId}/{auditState}")
    public ResultView haveDone(@PathVariable Integer userId, @PathVariable String type, @PathVariable Integer depotId,@PathVariable Integer auditState) {
        processAuditService.haveDone(userId,type,depotId,auditState);
        return ResultView.success();
    }


    @Autowired
    private SystemFeign systemFeign;

    /**
     * 根据待办id，将流程设成已办
     同时要存储具体办理时的时间，因为要计算该流程办理的周期(或计算逾期时间)
     * @param id
     */
    @ApiOperation(value = "testsssssssssssssssssss", notes = "")
    @GetMapping("/haveDonetest/{id}")
    public ResultView haveDonetest(@PathVariable Integer id) {
        SysUserDTO userById = systemFeign.getUserById(id);
        return ResultView.success(userById);
    }

    /**
     * 获取待办预警通知
     */
    @ApiOperation(value = "获取待办预警通知", notes = "")
    @PostMapping("/getNotBeDoneNotice")
    public List<Integer> getNotBeDoneNotice(@RequestParam("toBeDoneIds") String toBeDoneIds) {
        List<Integer> result = processAuditService.getNotBeDoneNotice(toBeDoneIds);
        return result;
    }
    
    
    /**
     * 药剂保管
     */
    @ApiOperation(value = "药剂保管", notes = "")
    @PostMapping("/drugDetailsAdd")
    public ResultView drugDetailsAdd(@RequestBody String parameter) throws JSONException{
		processAuditService.drugDetailsAdd(parameter);
        return ResultView.success();
    }
    

}
