package com.unissoft.ventilation.service.impl;

import java.util.ArrayList;
import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.unissoft.mapper.ProcessBasisMapper;
import com.unissoft.mapper.ProcessBasisSonMapper;
import com.unissoft.mapper.ProcessMainMapper;
import com.unissoft.model.ProcessBasis;
import com.unissoft.model.ProcessBasisSon;
import com.unissoft.model.ProcessMain;
import com.unissoft.ventilation.service.ProcessBasisService;


/**
 * <p>
 *  服务实现类
 * </p>
 *
 * @author My SunShine
 * @since 2020-12-07
 */
@Service
public class ProcessBasisServiceImpl extends ServiceImpl<ProcessBasisMapper, ProcessBasis> implements ProcessBasisService {

	@Autowired
    private ProcessBasisMapper processBasisMapper;
	
	@Autowired
    private ProcessBasisSonMapper processBasisSonMapper;
	
	@Autowired
    private ProcessMainMapper processMainMapper;
	
	/**
	 * 1、根据传过来的depotId和类型查询该库有没有设置审批人，如果没有的话则增加，有的话则修改（通过title对比，更改content的值）
	 * 2、查询主流程表，把有流程的数据id保存到审批人管理里面的main_id字段
	 */
	@Override
	public void add(String parameter) {
		//解析获取传过来的数据
		JSONObject jsStr = JSONObject.parseObject(parameter);
		Integer depotId = jsStr.getInteger("depot_id");
		String type = jsStr.getString("type");
		JSONArray jsonArray = jsStr.getJSONArray("data");
		JSONObject row = null;
		
		QueryWrapper<ProcessBasis> ew = new QueryWrapper<>();
        ew.eq("type", type);
		List<ProcessBasis> pList  = processBasisMapper.selectList(ew);
		List<Integer> intList = new ArrayList<Integer>();
		if(pList.size()>0){
			for (ProcessBasis basis : pList) {
				intList.add(basis.getId());
			}
		}
		
		
		//根据类型查询主流程中哪个流程下是有审批流程的
		 QueryWrapper<ProcessMain> ew1 = new QueryWrapper<>();
		 if(type.equals("xzjh")){
			 ew.like("title", "熏蒸方案");
		 }else if(type.equals("xzsq")){
			 ew.like("title", "熏蒸散气");
		 }else{
			 ew.eq("type", type); 
		 }
		 
	     ew.eq("exist_process", 0);//存在审批流程的主流程
	     List<ProcessMain> mainList = processMainMapper.selectList(ew1);
			
			
		//审批人表数据
		QueryWrapper<ProcessBasisSon> ew2 = new QueryWrapper<>();
        ew2.eq("depot_id", depotId);
        ew2.in("zid", intList);
        List<ProcessBasisSon> sonList = processBasisSonMapper.selectList(ew2);
        
        if(sonList.size()>0){//修改
        	for (int i = 0; i < sonList.size(); i++) {
        		if(!sonList.get(i).getContent().equals(0)){//不是保管员 修改设置的值
        			ProcessBasis processBasis = processBasisMapper.selectById(sonList.get(i).getZid());
        			for (int j = 0; j < jsonArray.size(); j++) {
            			row = jsonArray.getJSONObject(j); 
            			String title = (String) row.get("title");
    					Integer content = (Integer) row.get("content");
    					if(processBasis.getTitle().equals(title)){
    						sonList.get(i).setContent(content);
    						processBasisSonMapper.updateById(sonList.get(i));
    					}
            		}
        			
				}
        		
        	}
        }else{//新增
        	ProcessBasisSon processBasisSon = new ProcessBasisSon();
        	
        	for (int i = 0; i < pList.size(); i++) {
        		for (int j = 0; j < jsonArray.size(); j++) {
        			row = jsonArray.getJSONObject(j); 
        			String title = (String) row.get("title");
					String step = (String) row.get("step");
					Integer content = (Integer) row.get("content");
					if(pList.get(i).getTitle().equals(title)){
						processBasisSon.setZid(pList.get(i).getId());
						processBasisSon.setDepotId(depotId);
						processBasisSon.setContent(content);
						processBasisSonMapper.insert(processBasisSon);
					}
					
					if(title.equals(pList.get(i).getTitle()) && step.equals(pList.get(i).getStep())){
                        pList.get(i).setMainId(mainList.get(0).getId());
                    	processBasisMapper.updateById(pList.get(i));
					}
				}
        		
			}
        }
		
	   
	}

	
	/*@Override
	public List<ProcessBasis> getBasicProcess(String type,Integer mainId) {
		// TODO Auto-generated method stub
		QueryWrapper<ProcessBasis> ew = new QueryWrapper<>();
        ew.eq("type", type);
        ew.eq("main_id", mainId);
        ew.like("title", "审批");
		return processBasisMapper.selectList(ew);
	}*/


	@Override
	public List<ProcessBasis> getBasisByType(String type,Integer userId,Integer depotId) {
		// TODO Auto-generated method stub
		
		QueryWrapper<ProcessBasis> ew = new QueryWrapper<>();
		if(null != type){
			ew.eq("type", type);
		}
		List<ProcessBasis> list = processBasisMapper.selectList(ew);
		
		
		QueryWrapper<ProcessBasisSon> ew1 = new QueryWrapper<>();
		if(null != depotId){
			ew1.eq("depot_id", depotId);
		}
		if(list.size()>0){
		    
			List<Integer> catList = new ArrayList<>();
			for (ProcessBasis processBasis : list) {
				catList.add(processBasis.getId());
			}
			ew1.in("zid", catList);
		}
		List<ProcessBasisSon> sonList = processBasisSonMapper.selectList(ew1);
		if(list.size()>0){
			if(sonList.size()>0){
				for (int i = 0; i < list.size();i++) {
					for (int j = 0; j < sonList.size();j++){
						if(list.get(i).getId().equals(sonList.get(j).getZid())){
							list.get(i).setContent(sonList.get(j).getContent());
							list.get(i).setDepotId(sonList.get(j).getDepotId());
						}
					}
				}
			}else{//未设置审批人信息
				for (int i = 0; i < list.size();i++) {
					if(!list.get(i).getTitle().contains("审批")){
						list.get(i).setContent(0);
					}
				}
				
			}
			
		}
		
		return list;
	}

	

}
