package com.unissoft.ventilation.service.impl;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import com.unissoft.MyConstant;
import com.unissoft.feign.SystemFeign;
import com.unissoft.mapper.StatisticsWorkCompletionMapper;
import com.unissoft.model.StatisticsWorkCompletion;
import com.unissoft.result.PageParam;
import com.unissoft.utils.DateUtil;
import com.unissoft.ventilation.service.StatisticsWorkCompletionService;
import org.apache.commons.lang.StringUtils;
import org.json.JSONObject;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * <p>
 * 工作完成情况统计表 服务实现类
 * </p>
 *
 * @author root
 * @since 2021-02-26
 */
@Service
public class StatisticsWorkCompletionServiceImpl extends ServiceImpl<StatisticsWorkCompletionMapper, StatisticsWorkCompletion> implements StatisticsWorkCompletionService {

    @Autowired
    private StatisticsWorkCompletionMapper workCompletionMapper;

    @Autowired
    private SystemFeign systemFeign;

    @Override
    public IPage<StatisticsWorkCompletion> getByPage(PageParam pageParam) {
        Integer pageIndex = pageParam.getPageIndex();
        Integer pageSize = pageParam.getPageSize();
        String depotId = null, sTime = null, type = null, groupBy = null;
        if (!StringUtils.isEmpty(pageParam.getCondition())) {
            JSONObject object = new JSONObject(pageParam.getCondition());
            depotId = object.optString("depotId").trim();
            sTime = object.optString("sTime").trim();
            type = object.optString("type").trim();
            groupBy = object.optString("groupBy").trim();
        }

        Page page = null;
        if (pageIndex != MyConstant.ZERO && pageSize != MyConstant.ZERO) {
            page = new Page(pageIndex, pageSize);
        }
        List<Integer> allDepotChild = null;
        if (StringUtils.isNotBlank(depotId) && StringUtils.isBlank(type)) {
            //获取本库下所有库id
            allDepotChild = systemFeign.getAllDepotChild(Integer.valueOf(depotId));
        }
        //type不等于空代表查询一个库下面的统计
        if (StringUtils.isNotBlank(type)) {
            allDepotChild = Lists.newArrayList();
            allDepotChild.add(Integer.valueOf(depotId));
        }
        //默认获取上个月份
        if (StringUtils.isEmpty(sTime)) {
            sTime = DateUtil.getLastMonth();
        } else {
            sTime += MyConstant.BURDEN_ZERO;
        }
        IPage<StatisticsWorkCompletion> pageResult = null;
        if (groupBy.equals(MyConstant.ONE_STR)) {
            //分组列表
            pageResult = workCompletionMapper.getGroupPage(page, allDepotChild, sTime);
        } else {
            //单个库列表
            pageResult = workCompletionMapper.getPage(page, allDepotChild, sTime);
        }
        return pageResult;
    }

    @Override
    public Map<String, Object> getGroupByDepotId(Integer depotId, Integer homeType) {

        // 1.分公司领导首页 2.库主任及以上领导首页 3.员工首页
        //获取上个月份
        String lastMonth = DateUtil.getLastMonth();
        ArrayList<String> depotNames = Lists.newArrayList();
        ArrayList<Integer> sScores = Lists.newArrayList();
        if (homeType == 1) {
            //查詢
            List<StatisticsWorkCompletion> groupByDepotId = workCompletionMapper.getGroupByDepotId(null, lastMonth);
            if (groupByDepotId != null && groupByDepotId.size() > MyConstant.ZERO) {
                //获取所有库map
                Map<String, List<StatisticsWorkCompletion>> collect = groupByDepotId.stream().collect(Collectors.groupingBy(t -> t.getDepotName()));
                for (String key : collect.keySet()) {
                    depotNames.add(key);
                    Double asDouble = collect.get(key).stream().mapToInt(t -> t.getSScore()).average().getAsDouble();
                    sScores.add(asDouble.intValue());
                }
            }
        }
        if (homeType == 2) {
            //获取本库下所有库id
            List<Integer> allDepotChild = systemFeign.getAllDepotChild(depotId);
            List<StatisticsWorkCompletion> groupByDepotId = workCompletionMapper.getGroupByDepotId(allDepotChild, lastMonth);
            if (groupByDepotId != null && groupByDepotId.size() > MyConstant.ZERO) {
                //获取所有库map
                Map<String, List<StatisticsWorkCompletion>> collect = groupByDepotId.stream().collect(Collectors.groupingBy(t -> t.getDepotName()));
                for (String key : collect.keySet()) {
                    depotNames.add(key);
                    Double asDouble = collect.get(key).stream().mapToInt(t -> t.getSScore()).average().getAsDouble();
                    sScores.add(asDouble.intValue());
                }
            }
        }
        if (homeType == 3) {
            //获取本库下所有库id
            List<StatisticsWorkCompletion> groupByDepotId = workCompletionMapper.getGroupByDepotId(Lists.newArrayList(depotId), lastMonth);
            for (StatisticsWorkCompletion comp : groupByDepotId) {
                depotNames.add(comp.getPersonnelName());
                sScores.add(comp.getSScore());
            }
        }

        HashMap<String, Object> map = Maps.newHashMap();
        map.put("depotName", depotNames);
        map.put("series", sScores);
        return map;
    }
}
